module CyberCoach
  #
  # An AbstractResource can be read.
  # Sets up serialization and deserialization using JSON.
  #
  class AbstractResource < Object
    #
    # Installs utilities to handle HTTP request and response.
    # See {API doc}[http://www.rubydoc.info/github/jnunemaker/httparty] and
    # {examples}[https://github.com/jnunemaker/httparty/tree/master/examples].
    #
    include HTTParty

    #
    # Instance variables can be initialized with a hash.
    #
    include HashInitializable

    #
    # :private_attr: options
    # The options that are always sent with each request, e.g. :basic_auth.
    #

    #
    # The URI, the global identifier of it.
    #
    attr_accessor :uri

    headers 'content-type' => 'application/json', 'accept' => 'application/json'
    format :json
    base_uri Settings::SERVER_URI

    #
    # Creates an AbstractResource.
    # hash:: A hash of values to set on instance variables.
    #
    def initialize(hash = {})
      super()
      @options = {}
      initialize_with(hash)
    end

    #
    # :category: Invalidation
    #
    # Invalidates the uri and the options.
    #
    def invalidate
      invalidate_uri
      invalidate_options
    end

    #
    # :category: CRUD
    #
    # Reads it.
    # Gets the URI from the response and reads itself again.
    # Raises HttpError if the request is unsuccessful.
    # options:: A hash of options to send with the request.
    # invalidate:: Invalidates it when true, skips invalidation when false.
    #
    def read(options = {}, invalidate = true)
      if invalidate
        self.invalidate
      end
      options = @options.merge(options)
      response = self.class.get(@uri, options)
      if response.success?
        deserialize(response)
      else
        fail HttpError, response.response
      end
    end

    #
    # :category: Serialization
    #
    # Returns a text based representations.
    #
    def serialize
      format = self.class.default_options[:format]
      if format.nil?
        to_serializable
      elsif format == :json
        to_serializable.to_json
      else
        fail FormatNotSupportedError
      end
    end

    #
    # :category: Serialization
    #
    # Deserializes it from a text based representation.
    # serialization:: A text based representation.
    #
    def deserialize(serialization)
      from_serializable(serialization)
    end

    #
    # :category: Serialization
    #
    # Creates itself from a serializable representation, which only contains
    # simple data types.
    # serializable:: A hash with the keys:
    # * uri:: The URI.
    #
    def from_serializable(serializable)
      @uri = serializable['uri']
    end

    #
    # :category: Serialization
    #
    # Returns a serializable representation, which only contains simple data
    # types.
    # The hash has the keys:
    # * uri:: The URI.
    #
    def to_serializable
      serializable = {}
      serializable['uri'] = @uri
      serializable
    end

    #
    # :category: Configuration
    #
    # Returns the singular name of the resource.
    # E.g. 'user' or 'entry'.
    # This is used for parsing.
    # Must be overridden in a subclass.
    #
    def singular_name
      fail SubclassResponsibilityError
    end

    #
    # :category: Configuration
    #
    # Returns the plural name of the resource.
    # E.g. 'users' or 'entries'.
    # This is used for parsing.
    # Must be overridden in a subclass.
    #
    def plural_name
      fail SubclassResponsibilityError
    end

    #
    # :category: Configuration
    #
    # Returns the base URI relative to the server.
    #
    def resource_base_uri
      "#{Settings::BASE_URI}/#{plural_name}/"
    end

    protected

    #
    # :category: Invalidation
    #
    # Recalculates the URI from the attributes.
    # A template method called before a request is made.
    #
    def invalidate_uri
    end

    #
    # :category: Invalidation
    #
    # Recalculates the request options from the attributes.
    # A template method called before a request is made.
    #
    def invalidate_options
    end

    def initializable_with
      [:uri]
    end
  end
end
