module CyberCoach
  #
  # A Partnership consists of two Users, which participate in different Sport
  # with Subscriptions, to which Entries are submitted.
  #
  class Partnership < Resource
    #
    # It is pageable.
    #
    include Pageable

    #
    # It is creatable by PUT.
    #
    include PutCreateable

    #
    # :attr: proposer
    # The User who proposed it.
    #

    #
    # :attr: proposed
    # The User it is proposed to.
    #

    #
    # :attr: subscriptions
    # The Subscriptions.
    #

    #
    # :attr: confirmed_by_proposer
    # True if the proposing User has confirmed it, false otherwise.
    #

    #
    # :attr: confirmed_by_proposed
    # True if the proposed User has confirmed it, false otherwise.
    #

    #
    # :attr: privacy_level
    # The privacy level, see PrivacyLevel constants.
    #

    attr_accessor :proposer,
                  :proposed,
                  :subscriptions,
                  :confirmed_by_proposer,
                  :confirmed_by_proposed,
                  :privacy_level

    #
    # :category: Serialization
    #
    # Creates itself from a serializable representation, which only contains
    # simple data types.
    # serializable:: A hash with the keys:
    # * uri:: The URI.
    # * id:: The identifier.
    # * user1:: A User serializable of the proposer.
    # * user2:: A User serializable of the proposed.
    # * subscriptions:: Subscription serializables.
    # * userconfirmed1:: True if the proposing User has confirmed it, false otherwise.
    # * userconfirmed2:: True if the proposed User has confirmed it, false otherwise.
    # * publicvisible:: The privacy level, see PrivacyLevel constants.
    #
    def from_serializable(serializable)
      super(serializable)
      @proposer = nil
      unless serializable['user1'].nil?
        @proposer = User.new
        @proposer.from_serializable(serializable['user1'])
      end
      @proposed = nil
      unless serializable['user2'].nil?
        @proposed = User.new
        @proposed.from_serializable(serializable['user2'])
      end
      @subscriptions = []
      unless serializable['subscriptions'].nil?
        @subscriptions = serializable['subscriptions'].map do
        |subscription_serializable|
          subscription = Subscription.new
          subscription.from_serializable(subscription_serializable)
          subscription
        end
      end
      @confirmed_by_proposer = serializable['userconfirmed1']
      @confirmed_by_proposed = serializable['userconfirmed2']
      @privacy_level = serializable['publicvisible']
    end

    #
    # :category: Serialization
    #
    # Returns a serializable representation, which only contains simple data
    # types.
    # The hash has the keys:
    # * uri:: The URI.
    # * id:: The identifier.
    # * user1:: A User serializable of the proposer.
    # * user2:: A User serializable of the proposed.
    # * publicvisible:: The privacy level, see PrivacyLevel constants.
    #
    def to_serializable
      serializable = super
      unless @proposer.nil?
        serializable['user1'] = @proposer.to_serializable
      end
      unless @proposed.nil?
        serializable['user2'] = @proposed.to_serializable
      end
      serializable['publicvisible'] = @privacy_level
      serializable
    end

    #
    # :category: Configuration
    #
    # Returns 'partnership'.
    #
    def singular_name
      'partnership'
    end

    #
    # :category: Configuration
    #
    # Returns 'partnerships'.
    #
    def plural_name
      'partnerships'
    end

    protected

    #
    # :category: Invalidation
    #
    # Sets the uri to the base uri and the proposer's and proposed's username if
    # neither of them is nil.
    #
    def invalidate_uri
      unless @proposer.nil? || @proposed.nil?
        @uri = "#{resource_base_uri}#{@proposer.username};#{@proposed.username}/"
      end
    end

    def initializable_with
      super + [:proposer, :proposed, :privacy_level]
    end
  end
end
