module CyberCoach
  #
  # A Resource can be created, read, updated and deleted.
  #
  class Resource < AbstractResource
    #
    # The identifier.
    #
    attr_accessor :id

    #
    # :category: CRUD
    #
    # Creates it.
    # Must be overridden in a subclass.
    # May use PutCreateable or PostCreateable.
    # options:: A hash of options to send with the request.
    # invalidate:: Invalidates it when true, skips invalidation when false.
    #
    def create(options = {}, invalidate = true)
      fail SubclassResponsibilityError
    end

    #
    # :category: CRUD
    #
    # Updates it.
    # Reads itself from the response.
    # Raises HttpError if the request is unsuccessful.
    # options:: A hash of options to send with the request.
    # invalidate:: Invalidates it when true, skips invalidation when false.
    #
    def update(options = {}, invalidate = true)
      if invalidate
        self.invalidate
      end
      options = @options.merge(options).merge(
        body: serialize
      )
      response = self.class.put(@uri, options)
      if response.success?
        deserialize(response)
      else
        fail HttpError, response.response
      end
    end

    #
    # :category: CRUD
    #
    # Deletes it.
    # Reads itself from the response.
    # Raises HttpError if the request is unsuccessful.
    # options:: A hash of options to send with the request.
    # invalidate:: Invalidates it when true, skips invalidation when false.
    #
    def delete(options = {}, invalidate = true)
      if invalidate
        self.invalidate
      end
      options = @options.merge(options)
      response = self.class.delete(@uri, options)
      if response.success?
        deserialize(response)
      else
        fail HttpError, response.response
      end
    end

    #
    # :category: Serialization
    #
    # Creates itself from a serializable representation, which only contains
    # simple data types.
    # serializable:: A hash with the keys:
    # * uri:: The URI.
    # * id:: The identifier.
    #
    def from_serializable(serializable)
      super(serializable)
      @id = serializable['id']
    end

    #
    # :category: Serialization
    #
    # Returns a serializable representation, which only contains simple data
    # types.
    # The hash has the keys:
    # * uri:: The URI.
    # * id:: The identifier.
    #
    def to_serializable
      serializable = super
      serializable['id'] = @id
      serializable
    end

    def initializable_with
      super + [:id]
    end
  end
end
