module CyberCoach
  #
  # A ResourcePage can be used to navigate through many resources of a type.
  #
  class ResourcePage < AbstractResource
    #
    # Raised when trying to get the next page from the last page.
    #
    class NoNextPageError < StandardError
    end

    #
    # Raised when trying to get the previous page from the first page.
    #
    class NoPreviousPageError < StandardError
    end

    #
    # :attr: start
    # The start index.
    #

    #
    # :attr: end
    # The end index.
    #

    #
    # :attr: size
    # The size.
    #

    #
    # :attr: available
    # The resources available.
    #

    #
    # :attr: type
    # The class of the resource to page.
    #

    #
    # :attr: resources
    # The resources.
    #

    attr_accessor :start, :end, :size, :available, :type, :resources

    #
    # Create a ResourcePage of the specified type.
    # hash:: A hash of values to set on instance variables. :type is required.
    #
    def initialize(hash = {})
      unless hash.key? :type
        fail ArgumentError, 'requires a :type entry in the hash'
      end
      super(hash)
    end

    #
    # :category: CRUD
    #
    # Returns the next page.
    # Raises NoNextPageError if the is none.
    # options:: A hash of options to send with the request.
    # invalidate:: Invalidates it when true, skips invalidation when false.
    #
    def next(options = {}, invalidate = true)
      if @next.nil?
        fail NoNextPageError
      end
      if invalidate
        self.invalidate
      end
      options = @options.merge(options)
      response = self.class.get(@next['href'], options)
      if response.success?
        page = self.class.new(type: @type)
        page.deserialize(response)
        page
      else
        fail HttpError, response.response
      end
    end

    #
    # :category: CRUD
    #
    # Returns the previous page.
    # Raises NoPreviousPageError if the is none.
    # options:: A hash of options to send with the request.
    # invalidate:: Invalidates it when true, skips invalidation when false.
    #
    def previous(options = {}, invalidate = true)
      if @previous.nil?
        fail NoPreviousPageError
      end
      if invalidate
        self.invalidate
      end
      options = @options.merge(options)
      response = self.class.get(@previous['href'], options)
      if response.success?
        page = self.class.new(type: @type)
        page.deserialize(response)
        page
      else
        fail HttpError, response.response
      end
    end

    #
    # :category: Serialization
    #
    # Creates itself from a serializable representation, which only contains
    # simple data types.
    # serializable:: A hash with the keys:
    # * uri:: The URI.
    # * start:: The start index.
    # * end:: The end index.
    # * size:: The size.
    # * available:: The resources available.
    # * links:: Links as hashes with the keys:
    #   * description:: May be 'next' or 'previous'.
    #   * href:: The URI of the referenced page.
    # * @plural_name:: Items mapped to the plural name of the @type.
    #
    def from_serializable(serializable)
      super(serializable)
      @start = serializable['start']
      @end = serializable['end']
      @size = serializable['size']
      @available = serializable['available']
      if serializable['links'].nil?
        @next = nil
        @previous = nil
      else
        @next = serializable['links'].find { |link| link['description'] == 'next' }
        @previous = serializable['links'].find { |link| link['description'] == 'previous' }
      end
      @resources = serializable[plural_name].map do |resource_serializable|
        resource = @type.new
        resource.from_serializable(resource_serializable)
        resource
      end
    end

    #
    # :category: Configuration
    #
    # Return the singular name of the type.
    #
    def singular_name
      @type.new.singular_name
    end

    #
    # :category: Configuration
    #
    # Return the plural name of the type.
    #
    def plural_name
      @type.new.plural_name
    end

    #
    # :category: Configuration
    #
    # Return the resource's base URI of the type.
    #
    def resource_base_uri
      @type.new.resource_base_uri
    end

    protected

    #
    # :category: Invalidation
    #
    # Sets the start and size attributes as query parameters.
    #
    def invalidate_options
      @options[:query] = {
        start: @start,
        size: @size
      }
    end

    #
    # :category: Invalidation
    #
    # Sets the URI to the types' base URI.
    #
    def invalidate_uri
      @uri = resource_base_uri
    end

    def initializable_with
      super + [:type, :start, :end, :size]
    end
  end
end
