module CyberCoach
  #
  # A Subscription to a Sport is placed by a User or a Partnership and contains
  # Entries.
  #
  class Subscription < Resource
    #
    # It is creatable by PUT.
    #
    include PutCreateable

    #
    # :attr: subscriber
    # Either a User or a Partnership, which places Entries to it.
    #

    #
    # :attr: sport
    # The Sport.
    #

    #
    # :attr: entries
    # The Entries already placed by the subscriber.
    #

    #
    # :attr: privacy_level
    # The privacy level, see PrivacyLevel constants.
    #

    #
    # :attr: date_created
    # The date it was created.
    #

    attr_accessor :subscriber, :sport, :entries, :privacy_level, :date_created

    #
    # :category: Serialization
    #
    # Creates itself from a serializable representation, which only contains
    # simple data types.
    # serializable:: A hash with the keys:
    # * uri:: The URI.
    # * id:: The identifier.
    # * user|partnership:: A User or Partnership serializable of the subscriber.
    # * sport:: A Sport serializable.
    # * entries:: Entry serializables.
    # * publicvisible:: The privacy level, see PrivacyLevel constants.
    # * datesubscribed:: The date it was created.
    #
    def from_serializable(serializable)
      super(serializable)
      @subscriber = nil
      unless serializable['user'].nil?
        @subscriber = User.new
        @subscriber.from_serializable(serializable['user'])
      end
      unless serializable['partnership'].nil?
        @subscriber = Partnership.new
        @subscriber.from_serializable(serializable['partnership'])
      end
      @sport = nil
      unless serializable['sport'].nil?
        @sport = Sport.new
        @sport.from_serializable(serializable['sport'])
      end
      @entries = []
      unless serializable['entries'].nil?
        @entries = serializable['entries'].map do |entry_serializable|
          entry = Entry.new
          entry.from_serializable(entry_serializable)
          entry
        end
      end
      @privacy_level = serializable['publicvisible']
      @date_created = nil
      unless serializable['datesubscribed'].nil?
        @date_created = Time.at(serializable['datesubscribed']).to_datetime
      end
    end

    #
    # :category: Serialization
    #
    # Returns a serializable representation, which only contains simple data
    # types.
    # The hash has the keys:
    # * uri:: The URI.
    # * id:: The identifier.
    # * user|partnership:: A User or Partnership serializable of the subscriber.
    # * sport:: A Sport serializable.
    # * publicvisible:: The privacy level, see PrivacyLevel constants.
    #
    def to_serializable
      serializable = super
      serializable[@subscriber.singular_name] = @subscriber.to_serializable
      serializable['sport'] = @sport.to_serializable
      serializable['publicvisible'] = @privacy_level
      serializable
    end

    #
    # :category: Configuration
    #
    # Returns 'subscription'.
    #
    def singular_name
      'subscription'
    end

    #
    # :category: Configuration
    #
    # Returns 'subscriptions'.
    #
    def plural_name
      'subscriptions'
    end

    #
    # :category: Configuration
    #
    # Return the URI of its subscriber and sport.
    #
    def resource_base_uri
      "#{@subscriber.uri}#{@sport.name}/"
    end

    protected

    #
    # :category: Invalidation
    #
    # Sets the uri to the base uri if neither the subscriber, nor the sport is
    # nil.
    #
    def invalidate_uri
      unless @subscriber.nil? || @sport.nil?
        @uri = resource_base_uri
      end
    end

    def initializable_with
      super + [:subscriber, :sport, :privacy_level]
    end
  end
end
