module CyberCoach
  #
  # A User is needed to participate in Partnerships and Subscriptions, to which
  # Entries are submitted.
  #
  class User < Resource
    #
    # It is pageable.
    #
    include Pageable

    #
    # It is creatable by PUT.
    #
    include PutCreateable

    #
    # :attr: username
    # The username.
    #

    #
    # :attr: email
    # The email address.
    #

    #
    # :attr: password
    # The password, which is never received from the server.
    #

    #
    # :attr: real_name
    # The real name.
    #

    #
    # :attr: privacy_level
    # The privacy level, see PrivacyLevel constants.
    #

    #
    # :attr: date_created
    # The date it was created.
    #

    attr_accessor :username,
                  :email,
                  :password,
                  :real_name,
                  :privacy_level,
                  :date_created

    #
    # :category: Serialization
    #
    # Creates itself from a serializable representation, which only contains
    # simple data types.
    # serializable:: A hash with the keys:
    # * uri:: The URI.
    # * id:: The identifier.
    # * username:: The username.
    # * email:: The email address.
    # * realname:: The real name.
    # * publicvisible:: The privacy level, see PrivacyLevel constants.
    # * datecreated:: The date it was created.
    #
    def from_serializable(serializable)
      super(serializable)
      @username = serializable['username']
      @email = serializable['email']
      # the password is never received from the server, so don't set it
      @real_name = serializable['realname']
      @privacy_level = serializable['publicvisible']
      @date_created = nil
      unless serializable['datecreated'].nil?
        @date_created = Time.at(serializable['datecreated']).to_datetime
      end
    end

    #
    # :category: Serialization
    #
    # Returns a serializable representation, which only contains simple data
    # types.
    # The hash has the keys:
    # * uri:: The URI.
    # * id:: The identifier.
    # * username:: The username.
    # * email:: The email address.
    # * realname:: The real name.
    # * publicvisible:: The privacy level, see PrivacyLevel constants.
    # * datecreated:: The date it was created.
    #
    def to_serializable
      serializable = super
      serializable['username'] = @username
      serializable['email'] = @email
      serializable['password'] = @password
      serializable['realname'] = @real_name
      serializable['publicvisible'] = @privacy_level
      serializable
    end

    #
    # Returns the authentication options for basic auth, to pass in CRUD methods
    # of other resources.
    #
    def authentication
      {
        basic_auth: {
          username: @username,
          password: @password
        }
      }
    end

    #
    # :category: Configuration
    #
    # Returns 'user'.
    #
    def singular_name
      'user'
    end

    #
    # :category: Configuration
    #
    # Returns 'users'.
    #
    def plural_name
      'users'
    end

    protected

    #
    # :category: Invalidation
    #
    # Sets the uri to the base uri and the username, if it is not nil.
    #
    def invalidate_uri
      unless @username.nil?
        @uri = "#{resource_base_uri}#{@username}/"
      end
    end

    #
    # :category: Invalidation
    #
    # Sets the basic auth token to the login, so it can be updated and deleted
    # without explicitly specifying the token.
    #
    def invalidate_options
      @options = @options.merge(authentication)
    end

    def initializable_with
      super + [:username, :email, :password, :real_name, :privacy_level]
    end
  end
end
